	PROGRAM GRID_TO_VOX
C
C	Version 3.1
C	Modified from version 2, April 1, 2009 (Macintosh version)
C       to Version 3.1 (adapted to PC, 15.2.2011).
C
C	By Michael Knappertsbusch
C
C	Batch mode of operation
C
C	For Globorotalia menardii:
C	Conversion of gridded data to XYZF data for Voxler WITH
C       normalization for frequency:
C
C	Input is a 14x16 matrix U of gridded X,Y,Z,F data, which was
C       obtained as the output from program Grid.out.
C	The X and Y indicate the size of the grid-cell, i.e. - for
C       G. menardii in keel view - X stands for mid-intervals of
C       DeltaX (=spiral height) and Y stands for mid-intervals of
C       DeltaY (=axial length). Z is the geological age in million
C       years ago, and F is the frequency of specimens per grid-cell.
C       X is arranged in horizontal rows, Y is arranged in vertical
C       direction (columns). Values of F (i.e. the matrix numbers)
C       indicate the frequency in specimens per grid cell.
C
C	Output is a re-organized 4x224 matrix with normalized frequency
C       values of F (in % of the total number of specimens in that
C       sample), but with X (=mid-points of DeltaX) being in the first
C       column, Y (=mid-points of DeltaY in the second column, Z (=age)
C       in the third column, and F (=frequency, in %) in the fourth
C       column. This format can be imported to commercial program
C       Voxler (http://www.goldensoftware.com) in order to generate
C       animated 3D graphical representations of the data.
C
C	For reasons of good scaling of the X,Y and Z axes there are
C       three options for the output:
C
C	Option 1 (ANS=1) is with scaling of the axes.
C	  The X axis is divided by the maximum value of X (=675 or
C         Y(14) ), the Y axis is divided by the maximum value of Y
C         (=1550 or X(16) ), and the Z axis is divided by the maximum
C         age (ZMAX) given in List_of_files.
C	  The frequencies are written to file as per cent of the
C         specimen number in that sample. This way, the X,Y,Z values
C         are normalized to values between 0 and 1.
C
C	Option 2 (ANS=2) is without scaling of the X,Y and Z axes, and
C         only the frequencies are normalized (per cent of the
C         specimens per sample).
C
C	Option 3 (ANS=3) is without scaling of the X,Y and Z axes,
C         and without normalization of the frequencies (option 3
C         corresponds to version Grid_to_Vox1.out)
C
C
C
C	Additional options for the output:
C	The results can be written to external files in two ways:
C
C	Way 1 (BANS=1): For each individual sample input1xxxxx_grid
C       a result file is created (either with names of
C       input1xxxxx_XYZFsn, input1xxxxx_XYZFn, or input1xxxxx_XYZF
C       according to the previously chosen options 1,2 or 3.
C
C	Way 2 (BANS=2): One single result file is created with the
C       results of all treated input files pasted together. The result
C       file has the name ALL_XYZFsn, ALL_XYZFn or ALL_XYZF according
C       to the previously chosen options 1,2 or 3.
C
C
C
	INTEGER N,I,J,P,ANS,BANS
	INTEGER MOA,MOB
	REAL NSPEC           ! Number of specimens in a sample
	REAL F(1:14,1:16)    ! Input matrix containing Frequency values
	REAL AGE,MAXAGE      ! Age of sample, and maximum age of sample
C                              in input file list (in Ma)
	REAL X(1:16),Y(1:14)
	REAL ZMAX            ! Maximum age common to all cores
C                             (for scaling the Z axis in Voxler, in Ma)
	CHARACTER*17 LIST    ! Name of list containing the ages and
C                              the file names of the samples
	CHARACTER*24 INPUT   ! Name of file with gridded data
	CHARACTER*27 OUTPUT  ! Name of file with XYZF data for Voxler
	CHARACTER*1 TAB
C
	TAB=','
C
C	Midpoints of intervals of DeltaX (=spiral heigt) or DeltaY
C       (axial length):
C
	DATA (X(I),I=1,16)/50,150, ! Intervals for rows for G. menardii
     1  250,350,450,550,650,750,
     2  850,950,1050,1150,1250,
     3  1350,1450,1550/
C
C
C      This data bloc contains the intervals for columns (Diameters)
C      for C. leptoporus:
C
	DATA (Y(I),I=1,14)/25,75,125,  ! Intervals for cols for
C                                         G. menardii
     1  175,225,275,325,375,
     2  425,475,525,575,625,675/
C
C
C
	WRITE(*,*) '**************************************************'
	WRITE(*,*) '*                                                *'
	WRITE(*,*) '*               Grid_to_Vox3.1                   *'
	WRITE(*,*) '*      Preparing gridded data for Voxler         *'
	WRITE(*,*) '*                                                *'
	WRITE(*,*) '*  By Michael Knappertsbusch, February 15, 2011  *'
	WRITE(*,*) '* Version 3.1, batch processing operation mode   *'
	WRITE(*,*) '*                                                *'
	WRITE(*,*) '*       Adapted to PC, for G. menardii           *'
	WRITE(*,*) '*       With normalization of frequency          *'
	WRITE(*,*) '*                                                *'
	WRITE(*,*) '**************************************************'
C
C
C
C
C
	WRITE(*,*) '. . .Enter the list with samples. . .'
	WRITE(*,*) '    (max 17 chars including the'
	WRITE(*,*) '          appendix .txt'
	READ(*,3) LIST
 3	FORMAT(A17)
C
C
	WRITE(*,*) '. . .With scaling of X,Y,Z axes ?. . .'
	WRITE(*,*) 'Choose an option (1, 2 or 3)'
	WRITE(*,*) '1=with scaling X,Y,Z AND with normalization F'
	WRITE(*,*) '2=without scaling X,Y,Z but with normalization F'
	WRITE(*,*) '3=without scaling X,Y,Z AND
     * without normalization F'	
	READ(*,*) ANS
C
	WRITE(*,*) ' '
	WRITE(*,*) '. . .Options for output. . .'
	WRITE(*,*) 'Choose an option (1 or 2)'
	WRITE(*,*) '1=Separate file for each sample'
	WRITE(*,*) '2=All samples combined in one single file'
	READ(*,*) BANS
C
C	For scaling of axes determine the maximum age (=MAXAGE)
C       from the input list:
C
C      Determine the maximum age of the samples:
          MAXAGE=0.0                     ! Initialize MAXAGE
	  OPEN(13,FILE=LIST,STATUS='OLD')
C 	  READ(13,6) AGE,INPUT
C	  MAXAGE=AGE
 7	  READ(13,6,END=111) AGE,INPUT   ! Read next line
 	  IF (AGE.GT.MAXAGE) THEN
	    MAXAGE=AGE
	  END IF
	  GOTO 7
C
 111	  CLOSE(13)
          REWIND(13)
C
	WRITE(*,13) MAXAGE
 13	FORMAT('The maximum age is ',F6.3,' Ma')
C
C
C
	WRITE(*,*) '. . .Enter the maximum age common
     1 to all cores. . .'
	READ(*,*) ZMAX   ! ZMAX is used to scale the Z-axis in option 1
C
C
C       Main sample loop begins here:
C
C       Output file handling if option 2 is selected (output to
C       one single file):
C
 	IF (BANS.EQ.2) THEN
	  IF (ANS.EQ.1) THEN
	    OUTPUT='ALL_XYZFsn.txt'
	  ELSE IF (ANS.EQ.2) THEN
	    OUTPUT='ALL_XYZFn.txt'
	  ELSE IF (ANS.EQ.3) THEN
	    OUTPUT='ALL_XYZF.txt'
	  END IF
          OPEN(16,FILE=OUTPUT,STATUS='NEW')
	END IF
C
	P=0              ! Initialize counter for number of files
	OPEN(14,FILE=LIST,STATUS='OLD')
 5	READ(14,6,END=100) AGE,INPUT
 	P=P+1            ! Count number of files treated
 6	FORMAT(F6.3,1X,A24)
C
C       Output file handling, if option 1 is selected (output to
C       individual files):
C
        If (BANS.EQ.1) THEN
	  IF (ANS.EQ.1) THEN
	    OUTPUT=INPUT(1:16)//'_XYZFsn.txt'   ! Option 1
C                                (Scaling axes AND Normalization of F)
	  ELSE IF (ANS.EQ.2) THEN
	    OUTPUT=INPUT(1:16)//'_XYZFn.txt'    ! Option 2
C                                            (only Normalization of F)
	  ELSE IF (ANS.EQ.3) THEN
	    OUTPUT=INPUT(1:16)//'_XYZF.txt'     ! Option 3
C                      (neither scaling of axes nor normalization of F)
	  END IF
	  OPEN(16,FILE=OUTPUT,STATUS='NEW')
        END IF
C
C
C	Now the matrix manipulations for the activ input matrix begin.
C	Reading first the input matrix with Frequency values:
C
	OPEN(15,FILE=INPUT,STATUS='OLD')
	DO 10 J=1,16
	  READ(15,*) (F(I,J),I=1,14)
 10	CONTINUE
C
C
C	Calculate the sum (=NSPEC) of all elements in matrix F
C
	NSPEC=0.0
	DO 12 J=1,16
	  DO 11 I=1,14
	    NSPEC=NSPEC+F(I,J)   ! Calculates the number of specimens
C                                  in the sample
 11	  CONTINUE
 12	CONTINUE
C
C
C	Calculate indices and write results to output file:
C
C*****	Option 1:
C
	IF (ANS.EQ.1) THEN     ! Output with scaling of X, Y and Z
C                                axes and with normalization of F
	  DO 20 N=1,224        ! 224=14 columns * 16 rows
	    I=MOA(N)
	    J=MOB(N)
            WRITE(16,50) Y(J)/Y(14),TAB,X(I)/X(16),TAB,-AGE/ZMAX,
     1 TAB,100*F(J,I)/NSPEC
 20	  CONTINUE
 	END IF
C
 50	FORMAT(F6.4,A1,F6.4,A1,F7.3,A1,F6.2)
C
C
C*****	Option 2:
	IF (ANS.EQ.2) THEN    ! Output without scaling of X, Y and Z
C                               axes but with normalization of F
	  DO 30 N=1,224       ! 224=14 columns * 16 rows
	    I=MOA(N)
	    J=MOB(N)
	    WRITE(16,60) Y(J),TAB,X(I),TAB,-AGE,TAB,100*F(J,I)/NSPEC
 30	  CONTINUE
 	END IF
C
 60	FORMAT(F6.1,A1,F6.1,A1,F7.3,A1,F6.2)
C
C
C*****	Option 3:
	IF (ANS.EQ.3) THEN    ! Output without scaling of X, Y and Z
C                               and without normalization of F
	  DO 40 N=1,224       ! 224=14 columns * 16 rows
	    I=MOA(N)
	    J=MOB(N)
	    WRITE(16,70) Y(J),TAB,X(I),TAB,-AGE,TAB,F(J,I)
 40	  CONTINUE
 	END IF

 70	FORMAT(F6.1,A1,F6.1,A1,F7.3,A1,F6.2)
C
C
	CLOSE(15)            ! Closing active input matrix
	GOTO 5               ! Read age and name of next sample
C
 100	IF (BANS.EQ.2) THEN
 	  P=1
	END IF
	WRITE(*,101) P
 101	FORMAT('. . .',I3,' files treated. . .')
C
 	PAUSE 200
 200	CONTINUE
C
	STOP
	END
	







	
	INTEGER FUNCTION MOA(M)
C
C	Adapted for Voxler plots of G. menardii
C	Function to generate sequences of integer numbers like
C	1,2,3,...16,1,2,3,4,5,...,16,1,2,... as a function of N.
C
	INTEGER M
C
	IF (MOD(M,16).LT.1) THEN
C
C	A note on the above if-statement: The condition needs to be
C       Mod(M,16)=0; however, experimentation showed, that with the
C       modulo expression being set to zero the program did not return
C       the correct result (reason not fully understood): For M=16 or
C       integer multiples of 16 the program always returned a value
C       of 1+Int(M/16) instead of Int(M/16).
C	I think the reason is that the calculated modulo expression
C       does not return absolute zero but something like
C       0.00000000012357.... This means, that the if statement
C       Mod(M,16)=0 can never be met. Therefore, I use the above
C       alternative by using the if statement being Mod(M,16) < 1.
C
	  MOA=16
	ELSE
	  MOA=MOD(M,16)
	END IF
C
	RETURN
	END
	








	INTEGER FUNCTION MOB(M)
C
C	Adapted for Voxler plots of G. menardii
C	Function to generate sequences of integer numbers like
C	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
C	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
C	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
C	4,4,4,4,... as a function of N.
C
	INTEGER M
C
	IF (MOD(M,16).LT.1) THEN
C
C	A note on the above if-statement: The condition needs to be
C       Mod(M,16)=0; however, experimentation showed, that with the
C       modulo expression being set to zero the program did not return
C       the correct result (reason not fully understood): For M=16
C       or integer multiples of 16 the program always returned a value
C       of 1+Int(M/16) instead of Int(M/16).
C	I think the reason is that the calculated modulo expression
C       does not return absolute zero but something like
C       0.00000000012357.... This means, that the if statement
C       Mod(M,16)=0 can never be met. Therefore, I use the above
C       alternative by using the if statement being Mod(M,16) < 1.
C
	  MOB=   INT(M/16)
	ELSE
	  MOB= 1+INT(M/16)
	END IF
C
	RETURN
	END
	

    
